class Int2Printer:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        return "{%d,%d}" % (self.val['x'], self.val['y'])


class RangePrinter:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        return "{%s...%s}" % (self.val['start'], self.val['end'])


class RectPrinter:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        return "{pos = %s, size = %s}" % (self.val['pos'], self.val['size'])


class StringPrinter:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        size = int(self.val['chars']['i']['size'])
        if size == 0: return "<empty String>"
        return "{\"" + self.val['chars']['i']['data'].string('utf-8').replace('"', '\"') + \
            "\" size = %d" % size  + "}"


class ObjectPrinter:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        return "{instance of %s, refCount = %d}" % (self.val['classObj'], self.val['refCount'])


class PtrSetPrinter:
    def __init__(self, val):
        self.val = val

    def to_string(self):
        values = self.val['values']
        set_range = (int(values['range']['start']), int(values['range']['end']))
        set_size = set_range[1] - set_range[0]
        if set_size == 0: return "<empty PtrSet>"
        return "{size = %d}" % (set_size)


def lookup_type(val):
    type_str = str(val.type)
    if type_str.startswith('const '): type_str = type_str[6:]
    if type_str.endswith(' *'): type_str = type_str[:-2]
    if type_str == 'iString':
        return StringPrinter(val)
    if type_str.startswith('iRange'):
        return RangePrinter(val)
    if type_str == 'iInt2':
        return Int2Printer(val)
    if type_str == 'iRect':
        return RectPrinter(val)
    if type_str == 'iObject':
        return ObjectPrinter(val)
    if type_str == 'iPtrSet':
        return PtrSetPrinter(val)
    return None


gdb.pretty_printers.append(lookup_type)



import lldb

class PtrSet_SynthProvider:

    def __init__(self, valobj, dict):
        self.valobj = valobj
        self.update()

    def num_children(self):
        return self.elem_count

    def get_child_index(self, name):
        try:
            return int(name.lstrip('[').rstrip(']'))
        except:
            return -1

    def get_child_at_index(self, index):
        if index < 0:
            return None
        if index >= self.num_children():
            return None
        try:
            offset = (self.range_start.GetValueAsUnsigned(0) + index) * \
                self.valobj.GetProcess().GetAddressByteSize()
            return self.values_data.CreateValueFromExpression(
                '[' + str(index) + ']',
                '*(void**) &%s[%u]' % (self.values_data.get_expr_path(), offset)
            )
        except:
            return None

    def update(self):
        try:
            self.values_data = self.valobj.GetChildMemberWithName('values') \
                                          .GetChildMemberWithName('data')
            values_range = self.valobj.GetChildMemberWithName('values') \
                                      .GetChildMemberWithName('range')
            self.range_start = values_range.GetChildMemberWithName('start')
            self.range_end = values_range.GetChildMemberWithName('end')
            self.elem_count = self.range_end.GetValueAsUnsigned(0) \
                - self.range_start.GetValueAsUnsigned(0)            
        except Exception as x:
            print("Problem:", x)

    def has_children(self):
        return True


#def PtrSet_SummaryProvider(valobj, dict):
#    prov = PtrSet_SynthProvider(valobj, None)
#    return 'size=' + str(prov.num_children()) + " cmp=" + \
#        str(valobj.GetChildMemberWithName('cmd'))

